﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.VisualStudio.TestTools.UnitTesting.Logging;
using Oracle.ManagedDataAccess.Client;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace utPLSQL
{
    [TestClass]
    public class RealTimeTestRunnerTest
    {

        [TestMethod]
        public async Task TestRunTests()
        {
            var testRunner = new RealTimeTestRunner();

            testRunner.Connect(username: "ut3_tester", password: "ut3", database: "orclpdb1");

            var events = new List<@event>();
            await testRunner.RunTestsAsync("ut3_tester.test_ut_test", @event =>
            {
                events.Add(@event);
            });

            Assert.AreEqual("pre-run", events[0].type);
            Assert.AreEqual("post-run", events.Last().type);

            testRunner.Close();
        }

        [TestMethod]
        public async Task TestConnectAs()
        {
            var testRunner = new RealTimeTestRunner();

            testRunner.Connect(username: "sys", password: "Oradoc_db1", database: "orclpdb1", connectAs: "sysdba");

            try
            {
                await testRunner.RunTestsAsync("ut3_tester.test_ut_test", @event => { });

                Assert.Fail();
            }
            catch (OracleException e)
            {
                Assert.IsTrue(e.Message.StartsWith("ORA-06598"));

                testRunner.Close();
            }
        }

        [TestMethod]
        public async Task TestRunTestsWithCoverage()
        {
            var testRunner = new RealTimeTestRunner();

            testRunner.Connect(username: "ut3_tester", password: "ut3", database: "orclpdb1");

            var events = new List<@event>();

            string report = await testRunner.RunTestsWithCoverageAsync(path: "ut3_tester.test_ut_test", consumer: @event => { events.Add(@event); },
                                                                       coverageSchema: "ut3_develop", includeObjects: new List<string>() { "ut_test" });
            Logger.LogMessage(report);

            Assert.AreEqual("pre-run", events[0].type);
            Assert.AreEqual("post-run", events.Last().type);

            testRunner.Close();
        }

        [TestMethod]
        public void TestRunTestsAndAbort()
        {
            var testRunner = new RealTimeTestRunner();

            testRunner.Connect(username: "ut3_tester", password: "ut3", database: "orclpdb1");

            testRunner.RunTestsAsync("ut3_tester.test_ut_test", @event => { });

            testRunner.Close();
        }

        [TestMethod]
        public async Task TestRunTestsTwoTimes()
        {
            var testRunner = new RealTimeTestRunner();

            testRunner.Connect(username: "ut3_tester", password: "ut3", database: "orclpdb1");

            var events1 = new List<@event>();
            Task task1 = testRunner.RunTestsAsync("ut3_tester.test_ut_test", @event =>
            {
                events1.Add(@event);
            });

            var events2 = new List<@event>();
            Task task2 = testRunner.RunTestsAsync("ut3_tester.test_ut_test", @event =>
             {
                 events2.Add(@event);
             });

            await Task.WhenAll(task1, task2);

            testRunner.Close();
        }

        [TestMethod]
        public void TestGetVersion()
        {
            var testRunner = new RealTimeTestRunner();

            testRunner.Connect(username: "ut3_tester", password: "ut3", database: "orclpdb1");

            string version = testRunner.GetVersion();

            Assert.AreEqual("v3.1.11.3469-develop", version);

            testRunner.Close();
        }

        // [TestMethod] Disabled
        public void TestGetVersionWhenNotInstalled()
        {
            var testRunner = new RealTimeTestRunner();

            testRunner.Connect(username: "ut3_tester", password: "ut3", database: "orclpdb1");

            try
            {
                string version = testRunner.GetVersion();
                Assert.Fail();
            }
            catch (OracleException e)
            {
                Assert.AreEqual("ORA-00904: \"UT\".\"VERSION\": ungültige ID", e.Message);

                testRunner.Close();
            }
        }
    }
}
